<?php
/**
* This class shall help to build form data without doing html (or better: doing less html)
*
* Markus Bopp, 2003-11-06 01:50:00 CET
*/

class Form
{
	// private!
	var $name      = "";
	var $action    = "";
	var $method    = "";
	var $enctype   = "";
	var $extra     = "";
	var $openLine  = "<form[name][action][method][enctype][extra]>\n";
	var $elements  = array();
	var $closeLine = "</form>\n";
	
	function Form()
	{	
		// nothing yet ...
	}
	
	/**
	* Returns the closing tag of the form
	*
	*/
	function getCloseLine()
	{
		return $this->closeLine;
	}
	
	/**
	* Returns the opening tag of the form, including the properties
	*
	*/
	function getOpenLine()
	{
		if( strlen( $this->name ) != 0 )
		{
			$this_name = " name=\"" . $this->name . "\"";
		}else
		{
			$this_name = "";
		}
		
		if( strlen( $this->action ) != 0 )
		{
			$this_action = " action=\"" . $this->action . "\"";
		}else
		{
			$this_action = "";
		}
		
		if( strlen( $this->method ) != 0 )
		{
			$this_method = " method=\"" . $this->method . "\"";
		}else
		{
			$this_method = "";
		}
		
		if( strlen( $this->enctype ) != 0 )
		{
			$this_enctype = " enctype=\"" . $this->enctype . "\"";
		}else
		{
			$this_enctype = "";
		}
		
		if( strlen( $this->extra ) != 0 )
		{
			$this_extra = " " . $this->extra;
		}else
		{
			$this_extra = "";
		}
		
		$openLine2 = $this->openLine;
		
		$openLine2 = str_replace( "[name]",    $this_name,    $openLine2 );
		$openLine2 = str_replace( "[action]",  $this_action,  $openLine2 );
		$openLine2 = str_replace( "[method]",  $this_method,  $openLine2 );
		$openLine2 = str_replace( "[enctype]", $this_enctype, $openLine2 );
		$openLine2 = str_replace( "[extra]",   $this_extra,   $openLine2 );
		
		return $openLine2;
	}
	
	/**
	* Returns the element. If the return value of $this->elements is an array, it tries to
	* return the array. If an index > 0 is given, it returns the element stored at this position.
	* This should only be the case with radio fields.
	*
	*/
	function getElement( $name, $index = 0 )
	{
		if( isset( $this->elements[ $name ] ) )
		{
			if( is_array( $this->elements[ $name ] ) )
			{
				$elementsCount = count( $this->elements[ $name ] );
				for( $i = 0; $i < $elementsCount; $i++ )
				{
					if( $i == $index )
					{
						return $this->elements[ $name ][ $i ];
					}
				}
				
				if( $elementsCount == 1 )
				{
					return $this->elements[ $name ][ 0 ];
				}
				else
				{
					return $this->elements[ $name ];
				}
				
			}
			else
			{
				return $this->elements[ $name ];
			}
		}else
		{
			return "";
		}
	}
	
	/**
	* Returns the whole form as string. If there is a label given.
	* Like:
	*
	*	$label = array( "wipp" => "Wippe!", // non radio
	*			"sack123" => array( "value" => "furz", "sub_labels" => array( 0 => "peep", 1 => "poop" ) ) ); // if radio
	*
	* After the label there may be a breaking tag.
	* At least there can be a breaking tag been set after each value.
	*
	*/
	function toString( $label = array(), $labelBr = "", $br = "" )
	{
		$elements = "";
		$elements_length = count( $this->elements );
		
		$j = 0;
		
		while( list( $key, $value ) = each( $this->elements ) )
		{
			if( is_array( $value ) )
			{
				$sublabelCount = count( $value );
				
				if( isset( $label[ $key ] ) )
				{
					$elements .= $label[ $key ][ "value" ] . $labelBr;
					
					for( $i = 0; $i < $sublabelCount; $i++ )
					{
						$elements .= $label[ $key ][ "sub_labels" ][ $i ] . " ";
						$elements .= $this->getElement( $key, $i );
					}
					
					if( $j+1 < $elements_length )
					{
						$elements .= $br;
					}
				}
				else
				{
					for( $i = 0; $i < $sublabelCount; $i++ )
					{
						$elements .= $this->getElement( $key, $i ) . " ";
					}
					
					if( $j+1 < $elements_length )
					{
						$elements .= $br;
					}
				}
			}
			else
			{
				if( isset( $label[ $key ] ) )
				{
					$elements .= $label[ $key ] . $labelBr . $value;
					
					if( $j+1<$elements_length )
					{
						$elements .= $br;
					}
				}
				else
				{
					$elements .= $value;
					
					if( $j+1 < $elements_length )
					{
						$elements .= $br;
					}
				}
			}
			
			$j++;
		}
		
		return $this->getOpenLine() . $elements . $this->getCloseLine();
	}
	
	/**
	* Adds checkbox to the form.
	*
	*/
	function addCheckbox( $name, $value = "", $checked = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		if( strlen( $checked ) != 0 && $checked == "checked" )
		{
			$checked = " " . $checked;
		}else
		{
			$checked = "";
		}
		
		$this->elements[ $name ] = "<input type=\"checkbox\" name=\"$name\" value=\"$value\"{$extra}{$checked} />\n";
	}
	
	/**
	* Adds a radio to the form. If there already is a radio with the same name it appends it to the next index.
	* By giving the index in getElement( $name, $index = 0 ) you may access an element at the given index > 0.
	*
	*/
	function addRadio( $name, $value = "", $checked = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		if( strlen( $checked ) != 0 && $checked == "checked" )
		{
			$checked = " " . $checked;
		}else
		{
			$checked = "";
		}
		
		if( isset( $this->elements[ $name ] ) )
		{
			$this->elements[ $name ][ count( $this->elements[ $name ] ) ] = "<input type=\"radio\" name=\"$name\" value=\"$value\"{$extra}{$checked} />\n";
		}
		else
		{
			$this->elements[ $name ] = array( 0 => "<input type=\"radio\" name=\"$name\" value=\"$value\"{$extra}{$checked} />\n" );
		}
	}
	
	/**
	* Adds a submit button to the form.
	* If $image is given (source), it will show it as type="image"
	*
	*/
	function addSubmit( $name = "", $value = "", $image = "", $extra = "" )
	{
		$type = "submit";
		
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		if( strlen( $image ) != 0 )
		{
			$image = " src=\"$image\"";
			$type = "image";
		}
		
		if( strlen( $name ) != 0 )
		{
			$rname = $name;
			$name = " name=\"$name\"";
		}
		
		$this->elements[ $rname ] = "<input type=\"$type\" value=\"$value\"{$name}{$image}{$extra} />\n";
	}
	
	/**
	* Adds a select box to the form.
	* $value has to be an array like:
	*
	* $options = array( "number1" => 
	*				array( "value" => "number1 value", 
	*						"selected" => "selected",
	*						"extra" => "style=\"font-size:8pt;\"" ),
	*			  "number2" =>
	*			    array( "value" => "number2_value" ) );
	*
	* $form->addSelect( "mySelect", $options );
	*
	* By giving $multiple = "multiple" it adds a multiple select area.
	*
	*/
	function addSelect( $name, $value = array(), $multiple = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		if( strlen( $multiple ) != 0 && $multiple == "multiple")
		{
			$multiple = " " . $multiple;
		}
		else
		{
			$multiple = "";
		}
		
		$select = "<select name=\"$name\"{$extra}{$multiple}>\n";
	
		$valueCount = count( $value );
		while( list( $k, $v ) = each( $value ) )
		{
			if( !isset( $v["value"] ) )
			{
				$optionsValue = "";
			}else
			{
				$optionsValue = $v["value"];
			}
			
			if( !isset( $v["selected"] ) )
			{
				$selectedValue = "";
			}else
			{
				$selectedValue = " " . $v["selected"];
			}
			
			if( !isset( $v["extra"] ) )
			{
				$extraValue = "";
			}else
			{
				$extraValue = " " . $v["extra"];
			}
			
			$select .= "<option value=\"" . str_replace( "\n", "", str_replace( "\r", "", $k ) ) . "\"{$selectedValue}{$extraValue}>$optionsValue</option>\n";
		}

		$select .= "</select>\n";
		
		$this->elements[ $name ] = $select;
	}
	
	/**
	* Adds a textarea to the form.
	*
	*/
	function addTextarea( $name, $value = "", $cols = "", $rows = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		if( strlen( $cols ) != 0 )
		{
			$cols = " cols=\"" . $cols . "\"";
		}
		
		if( strlen( $rows ) != 0 )
		{
			$rows = " rows=\"" . $rows . "\"";
		}
		
		$this->elements[ $name ] = "<textarea name=\"$name\"{$cols}{$rows}{$extra}>".htmlspecialchars( $value )."</textarea>\n";
	}
	
	/**
	* Adds a text field to the form.
	*
	*/
	function addText( $name, $value = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		$this->elements[ $name ] = "<input type=\"text\" name=\"$name\" value=\"" . htmlspecialchars( $value ) . "\"{$extra} />\n";
	}
	
	function addFile( $name, $value = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		$this->elements[ $name ] = "<input type=\"file\" name=\"$name\" value=\"" . htmlspecialchars( $value ) . "\"{$extra} />\n";
	}
	
	/**
	* Adds a hidden field to the form.
	*
	*/
	function addHidden( $name, $value = "", $extra = "" )
	{
		if( strlen( $extra ) != 0 )
		{
			$extra = " " . $extra;
		}
		
		$this->elements[ $name ] = "<input type=\"hidden\" name=\"$name\" value=\"$value\"{$extra} />\n";
	}
	
	/**
	* Form properties setter methods
	* 
	*/
	function setName( $name )
	{
		$this->name = $name;
	}
	
	function setAction( $action )
	{
		$this->action = $action;
	}
	
	function setMethod( $method )
	{
		$this->method = $method;
	}
	
	function setEnctype( $enctype )
	{
		$this->enctype = $enctype;
	}
	
	function setExtra( $extra )
	{
		$this->extra = $extra;
	}
	
	/**
	* Form properties getter methods
	*/
	function getName()
	{
		return $this->name;
	}
	
	function getAction()
	{
		return $this->action;
	}
	
	function getMethod()
	{
		return $this->method;
	}
	
	function getEnctype()
	{
		return $this->enctype;
	}
	
	function getExtra()
	{
		return $this->extra;
	}
	
	function removeElement( $name )
	{
		unset( $this->elements[ $name ] );
	}
}
?>
